/*
 * Copyright (c) 2016, Oracle and/or its affiliates. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; version 2 of the
 * License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 */

#include <string>

/*
 * IMPORTANT: This file is autogenerated, do not change the code on this
 * file to properly do updates on this file please change the code at
 * <branch_root>/modules/adminapi/metadata-model.sql
*/

namespace shcore
{
  std::string md_model_sql="CREATE DATABASE mysql_innodb_cluster_metadata;\n"
"USE mysql_innodb_cluster_metadata;\n"
"\n"
"/*\n"
"  The major, minor and patch version of the schema representing the semantic\n"
"  version of the schema that is in use\n"
"*/\n"
"CREATE VIEW schema_version (major, minor, patch) AS SELECT 1, 0, 1;\n"
"\n"
"/*\n"
"  This table contain information about the metadata and is used to identify\n"
"  basic information about the cluster.\n"
"*/\n"
"CREATE TABLE clusters (\n"
"  /* unique ID used to distinguish the cluster from other clusters */\n"
"  `cluster_id` INT UNSIGNED PRIMARY KEY AUTO_INCREMENT,\n"
"  /* unique, user specified name for the cluster */\n"
"  `cluster_name` VARCHAR(40) UNIQUE NOT NULL,\n"
"  /* Points to the default replicaset. */\n"
"  `default_replicaset` INT UNSIGNED,\n"
"  /* Brief description of the cluster. */\n"
"  `description` TEXT,\n"
"  /*\n"
"    Stores default mysql user accounts for automated management.\n"
"    The passwords are encrypted using the FarmAdminPassword.\n"
"    {\n"
"      -- User for Read-Only access to the Farm (Router metadata cache access)\n"
"      mysqlRoUserName: <..>,\n"
"      mysqlRoUserPassword: <..>\n"
"    }\n"
"\n"
"    The data is in JSON format, but encrypted using the master key.\n"
"  */\n"
"  `mysql_user_accounts` BLOB,\n"
"  /*\n"
"    Stores all management options in the JSON format.\n"
"    {\n"
"      farmAdminType: < \"manual\" | \"ssh\" >,\n"
"      defaultSshUserName: < ... >,\n"
"      ...\n"
"    }\n"
"  */\n"
"  `options` JSON,\n"
"   /*\n"
"    Contain attributes assigned to each cluster and is a JSON data type with\n"
"    key-value pair. The attributes can be used to tag the clusters with custom\n"
"    attributes.\n"
"   */\n"
"  `attributes` JSON\n"
"\n"
") CHARSET = utf8mb4;\n"
"\n"
"/*\n"
"  The high-availability Replica Sets are the containers of the application data.\n"
"  Each group is composed of a number of MySQL servers in an HA configuration.\n"
"*/\n"
"CREATE TABLE replicasets (\n"
"  /*\n"
"    The replicaset_id is generated on creation of the replicaset and does not\n"
"    change over its lifetime.\n"
"  */\n"
"  `replicaset_id` INT UNSIGNED PRIMARY KEY AUTO_INCREMENT,\n"
"  /* Associates the replicaset with a cluster definition. */\n"
"  `cluster_id` INT UNSIGNED NOT NULL,\n"
"  /*\n"
"    Specifies the type of a replicaset, for now this needs to be set to\n"
"    Group Replication.\n"
"  */\n"
"  `replicaset_type` ENUM('gr') NOT NULL,\n"
"  /*\n"
"    Specifies the type of topology of a replicaset.\n"
"  */\n"
"  `topology_type` ENUM('pm', 'mm') NOT NULL DEFAULT 'pm',\n"
"  /*\n"
"    A replicaset can be assigned a name and this can be used to refer to it.\n"
"    The name of the replicaset can change over time and can even be NULL.\n"
"  */\n"
"  `replicaset_name` VARCHAR(40) NOT NULL,\n"
"  /*\n"
"    State of the rs. Either active, in which case traffic can be directed at\n"
"    the replicaset, or inactive, in which case no traffic should be directed\n"
"    to it.\n"
"  */\n"
"  `active` BOOLEAN NOT NULL,\n"
"  /*\n"
"    Custom properties.\n"
"    {\n"
"      groupReplicationChannelName: \"254616cc-fb47-11e5-aac5\"\n"
"    }\n"
"  */\n"
"  `attributes` JSON,\n"
"  /* An optional brief description of the replicaset. */\n"
"  `description` TEXT,\n"
"  FOREIGN KEY (cluster_id) REFERENCES clusters(cluster_id) ON DELETE RESTRICT\n"
") CHARSET = utf8mb4;\n"
"\n"
"ALTER TABLE clusters ADD FOREIGN KEY (default_replicaset) REFERENCES replicasets(replicaset_id) ON DELETE RESTRICT;\n"
"\n"
"/*\n"
"  This table contains a list of all the hosts in the cluster.\n"
"*/\n"
"CREATE TABLE hosts (\n"
"  /*\n"
"    The ID of the host instance. The host UUID is used internally for cluster\n"
"    management.\n"
"  */\n"
"  `host_id` INT UNSIGNED PRIMARY KEY AUTO_INCREMENT,\n"
"  /* network host name address of the host */\n"
"  `host_name` VARCHAR(256),\n"
"  /* network ip address of the host */\n"
"  `ip_address` VARCHAR(45),\n"
"  /* public network ip address of the host */\n"
"  `public_ip_address` VARCHAR(45),\n"
"  /* A string representing the location (e.g. datacenter name). */\n"
"  `location` VARCHAR(256) NOT NULL,\n"
"  /*\n"
"    Contain attributes assigned to the server host and is a JSON data type with\n"
"        key-value pair. The attributes can be used to tag the servers with custom\n"
"        attributes.\n"
"  */\n"
"  attributes JSON,\n"
"  /*\n"
"    Stores the admin user accounts information (e.g. for SSH) for automated\n"
"    management.\n"
"    {\n"
"      -- SSH User for Administrative access to the Host\n"
"      sshUserName: <..>\n"
"    }\n"
"  */\n"
"  `admin_user_account` JSON\n"
") CHARSET = utf8mb4;\n"
"\n"
"/*\n"
"  This table contain a list of all server instances that are tracked by the cluster.\n"
"*/\n"
"CREATE TABLE instances (\n"
"  /*\n"
"    The ID of the server instance and is a unique identifier of the server\n"
"    instance.\n"
"  */\n"
"  `instance_id` INT UNSIGNED PRIMARY KEY AUTO_INCREMENT,\n"
"  /* The ID of the host in which the server is running. */\n"
"  `host_id` INT UNSIGNED NOT NULL,\n"
"  /*\n"
"    Replicaset ID that the server belongs to. Can be NULL if it does not belong\n"
"    to any.\n"
"  */\n"
"  `replicaset_id` INT UNSIGNED,\n"
"  /* MySQL generated server_uuid for the instance */\n"
"  `mysql_server_uuid` VARCHAR(40) UNIQUE NOT NULL,\n"
"  /* unique, user specified name for the server */\n"
"  `instance_name` VARCHAR(256) UNIQUE NOT NULL,\n"
"  /* The role of the server instance in the setup e.g. scale-out, master etc. */\n"
"  `role` ENUM('HA', 'readScaleOut') NOT NULL,\n"
"  /*\n"
"    The mode of the server instance : if it accept read queries, read and write\n"
"    queries, or neither.\n"
"  */\n"
"  /*\n"
"    TODO: We don't store status information. Shouldn't this be the ReplicaSet mode?\n"
"    `mode` ENUM('rw', 'ro', 'none') NOT NULL,\n"
"  */\n"
"  /*\n"
"    The weight of the server instance for load balancing purposes. The relative\n"
"    proportion of traffic it should receive.\n"
"  */\n"
"  `weight` FLOAT,\n"
"  /*\n"
"    A JSON blob with the addresses available for the server instance. The\n"
"    protocols and addresses are further described in the Protocol section below.\n"
"    {\n"
"      mysqlClassic: \"mysql://host.foo.com:3306\",\n"
"      mysqlX: \"mysqlx://host.foo.com:33060\",\n"
"      localClassic: \"mysql://localhost:/tmp/mysql.sock\",\n"
"      localX: \"mysqlx://localhost:/tmp/mysqlx.sock\",\n"
"      mysqlXcom: \"mysqlXcom://host.foo.com:49213?channelName=<..>\"\n"
"    }\n"
"  */\n"
"  `addresses` JSON NOT NULL,\n"
"  /*\n"
"    Contain attributes assigned to the server and is a JSON data type with\n"
"    key-value pair. The attributes can be used to tag the servers with custom\n"
"    attributes.\n"
"  */\n"
"  `attributes` JSON,\n"
"  /*\n"
"    Server version token in effect for the server instance. The version token\n"
"    changes whenever there is a change of the role of the server instance and is\n"
"    used to force cache invalidation when topology changes.\n"
"  */\n"
"  `version_token` INTEGER UNSIGNED,\n"
"  /* An optional brief description of the group. */\n"
"  `description` TEXT,\n"
"  FOREIGN KEY (host_id) REFERENCES hosts(host_id) ON DELETE RESTRICT,\n"
"  FOREIGN KEY (replicaset_id) REFERENCES replicasets(replicaset_id) ON DELETE SET NULL\n"
") CHARSET = utf8mb4;\n"
"\n"
"\n"
"/*\n"
"  This table contain a list of all router instances that are tracked by the cluster.\n"
"*/\n"
"CREATE TABLE routers (\n"
"  /*\n"
"    The ID of the router instance and is a unique identifier of the server\n"
"    instance.\n"
"  */\n"
"  `router_id` INT UNSIGNED PRIMARY KEY AUTO_INCREMENT,\n"
"  /*\n"
"    A user specified name for an instance of the router.\n"
"   */\n"
"  `router_name` VARCHAR(256) NOT NULL,\n"
"  /* The ID of the host in which the server is running. */\n"
"  `host_id` INT UNSIGNED NOT NULL,\n"
"  /*\n"
"    Router specific custom attributes.\n"
"   */\n"
"  `attributes` JSON,\n"
"  FOREIGN KEY (host_id) REFERENCES hosts(host_id) ON DELETE RESTRICT,\n"
"  UNIQUE (host_id, router_name)\n"
") CHARSET = utf8mb4;\n"
"";
}
